import { OracleDbInterface } from '../Oracledb/OracleDbInterface';
import { TagCacheInterface } from '../Cache/CacheInterface';
import oracledb from 'oracledb';
export declare abstract class AbstractRepository {
    constructor({ oracleDb, cache }: AbstractRepositoryParameters);
    abstract readonly repositoryDomain: string;
    private readonly oracleDb;
    private readonly cache;
    /**
     * Returns an oracledb connection. Don't forget to close it when your're done :)
     */
    getConnection(): Promise<oracledb.Connection>;
    /**
     * Wrapper for oracledb.connection.execute that handles connection opening / closing
     * and query result caching. It Gets a connection, executes a query,
     * closes the connection and return results. The first three arguments are passed
     * to oracledb.connection.execute as provided, while the fourth argument controls
     * request caching ttl. The default ttl is 0, which means no caching is used.
     * Returns oracledb.Result.rows
     */
    execute<T>(sql: string, bindParams: oracledb.BindParameters, options: oracledb.ExecuteOptions, cachingOptions?: CachingOptions): Promise<T[]>;
}
interface CachingOptions {
    ttlInSecs?: number;
    cacheTags?: string[];
    responseTimeoutInMillis?: number;
    throwOnTimeout?: boolean;
}
interface AbstractRepositoryParameters {
    oracleDb: OracleDbInterface;
    cache: TagCacheInterface;
}
export {};
