"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.AbstractRepository = void 0;
const Errors_1 = require("../Errors");
const autobind_decorator_1 = require("autobind-decorator");
const lodash_1 = require("lodash");
const sha1_1 = __importDefault(require("sha1"));
class AbstractRepository {
    constructor({ oracleDb, cache }) {
        this.oracleDb = oracleDb.getDriver();
        this.cache = cache;
    }
    /**
     * Returns an oracledb connection. Don't forget to close it when your're done :)
     */
    async getConnection() {
        const connection = await this.oracleDb.getConnection();
        if (!connection)
            throw new Errors_1.InternalServerError('Invalid connection');
        return connection;
    }
    /**
     * Wrapper for oracledb.connection.execute that handles connection opening / closing
     * and query result caching. It Gets a connection, executes a query,
     * closes the connection and return results. The first three arguments are passed
     * to oracledb.connection.execute as provided, while the fourth argument controls
     * request caching ttl. The default ttl is 0, which means no caching is used.
     * Returns oracledb.Result.rows
     */
    async execute(sql, bindParams, options, cachingOptions) {
        const ttl = lodash_1.get(cachingOptions, 'ttlInSecs', 0);
        const cacheTags = lodash_1.get(cachingOptions, 'cacheTags', []);
        const responseTimeoutInMillis = lodash_1.get(cachingOptions, 'responseTimeoutInMillis');
        const throwOnTimeout = lodash_1.get(cachingOptions, 'throwOnTimeout');
        const querySignature = sha1_1.default(this.repositoryDomain + JSON.stringify({ sql, bindParams, options }));
        if (ttl > 0) {
            const cached = await this.cache.get(querySignature);
            if (cached)
                return cached;
        }
        const connection = await this.getConnection();
        const result = await connection.execute(sql, bindParams, options);
        const rows = result.rows;
        connection.close();
        if (ttl > 0) {
            await this.cache.set(querySignature, rows, cacheTags, {
                responseTimeoutInMillis,
                throwOnTimeout,
                ttlInSecs: ttl,
            });
        }
        return result.rows;
    }
}
__decorate([
    autobind_decorator_1.boundMethod
], AbstractRepository.prototype, "getConnection", null);
__decorate([
    autobind_decorator_1.boundMethod
], AbstractRepository.prototype, "execute", null);
exports.AbstractRepository = AbstractRepository;
